/*
 * embuttons.js
 */

var gPref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

// Replace BrowserOpenExtensions: Tools menu items will honour the EMbuttons settings

BrowserOpenExtensions = embuttonsOpenExtensions;

// Add the keyboard shortcut definitions to the Tools menu items (called by the onload event handler)

function embAddKeys()
{
   var allMenuItems = document.getElementsByTagName("menuitem");
   var i;
   for (i = 0; i < allMenuItems.length; ++i) {
      if (allMenuItems[i].getAttribute("command") == "Tools:Extensions") {
         allMenuItems[i].setAttribute("key","key_emb_extensions");
      }
      if (allMenuItems[i].getAttribute("command") == "Tools:Themes") {
         allMenuItems[i].setAttribute("key","key_emb_themes");
      }
   }
}

// Open the requested manager in the required manner

function embuttonsOpenExtensions(aOpenMode)
{

   const OPEN_WINDOW = 0;
   const OPEN_TAB = 1;
   const OPEN_SIDEBAR = 2;
   const EMTYPE = "Extension:Manager";
   const EMFEATURES = "chrome,dialog=no,resizable";
   const EMURL = "chrome://mozapps/content/extensions/extensions.xul?type=" + aOpenMode;
   const anIcon = "chrome://embuttons/skin/images/" + aOpenMode + "-small.png";
   const aSidebar = aOpenMode + 'EMbSidebar';

   var sidebarElt = document.getElementById(aSidebar);
   var aType = (aOpenMode == 'extensions') ? 'extension' : 'theme';

   var embBundle = document.getElementById("embstrings");
   var embError = embBundle.getString("embErrorCaption");
   var extTitle = embBundle.getString("embExtensionsTitle");
   var thmTitle = embBundle.getString("embThemesTitle");

   var panelTitle = (aOpenMode == 'extensions') ? extTitle : thmTitle;

   try {
      var openinPref = (aOpenMode == 'extensions') ? (gPref.getIntPref("extensions.embuttons.extensionsTarget")) : (gPref.getIntPref("extensions.embuttons.themesTarget"));
      var sortPref = (aOpenMode == 'extensions') ? (gPref.getBoolPref("extensions.embuttons.alphasortEM")) : (gPref.getBoolPref("extensions.embuttons.alphasortTM"));
   } catch(e){
      alert(embError);
      window.openDialog('chrome://embuttons/content/pref-embuttons.xul');
      return;
   }

// Check for the matching window - if it's open, focus it and don't try to open another

   var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                     .getService(Components.interfaces.nsIWindowMediator);
   var needToOpen = true;
   var windowType = EMTYPE + "-" + aOpenMode;
   var windows = wm.getEnumerator(windowType);
   while (windows.hasMoreElements()) {
      var theEM = windows.getNext().QueryInterface(Components.interfaces.nsIDOMWindowInternal);
      if (theEM.document.documentElement.getAttribute("windowtype") == windowType) {
         theEM.focus();
         needToOpen = false;
         break;
      }
   }

// Check for the matching tab - if it's open, focus it and avoid sorting - adapted from Jed Brown's QuickNote.
// Need to look at this at some point, it could be simplified. I think all I really need to check is currentURI.spec
// which should always match my EMURL. Would then also focus a manager which was opened with a bookmark etc.

   var index = 0;
   var numTabs = getBrowser().mTabContainer.childNodes.length;
   while (index < numTabs) {
      var tempTab = getBrowser().mTabContainer.childNodes[index];

      if (tempTab.getAttribute(aOpenMode) == 'true') {
         if (tempTab.label == EMURL){
            getBrowser().selectedTab = tempTab;
            needToOpen = false;
            sortPref = false;
            break;
         }else {
            getBrowser().selectedTab = tempTab;
            var url = getWebNavigation().currentURI.spec;
            if (url == EMURL) {
               needToOpen = false;
               sortPref = false;
               break;
            }
            else tempTab.removeAttribute(aOpenMode);
         }
      }
      else index = index+1;
   }

   if (needToOpen) {

// Check for the matching sidebar - if it's open, make sure it gets toggled closed and avoid sorting

      if (sidebarElt.getAttribute("checked") == "true") {
         openinPref = OPEN_SIDEBAR;
         sortPref = false;
      }

// Sort and count the extensions or themes (depending on the sort preference of course).
// Using the "embsort" attribute of the extensions sidebar broadcaster to make sure we don't try to do more than one sort
// simultaneously and risk a crash. Thanks to wig_out_on_me.

      var sortCheck = document.getElementById("extensionsEMbSidebar");
      if ((sortPref) && (sortCheck.getAttribute("embsort") != "true")) {
         sortCheck.setAttribute("embsort", "true");
         var extCount = embSortAndCount(aType,true);
         sortCheck.setAttribute("embsort", "false");
      }
      else var extCount = embSortAndCount(aType,false);

// Save count to be picked up by the window event handlers

      sidebarElt.setAttribute("count", extCount);

// Open the manager in the prescribed manner, or toggle the sidebar

      switch (openinPref) {
         case OPEN_WINDOW :
            window.openDialog(EMURL, "", EMFEATURES);
            break;
         case OPEN_TAB :
            var newTab = window.getBrowser().addTab(EMURL, null, null);
            newTab.setAttribute(aOpenMode,'true');
            newTab.label = panelTitle + ' [' + extCount + ']';
            newTab.setAttribute("image",anIcon);
            window.getBrowser().selectedTab = newTab;
            break;
         case OPEN_SIDEBAR :
            var oldTitle = sidebarElt.getAttribute("sidebartitle");
            var countPos = oldTitle.indexOf("[");
            if (countPos != -1) oldTitle = oldTitle.slice(0,--countPos);
            sidebarElt.setAttribute("sidebartitle",oldTitle + " [" + extCount + "]");
            toggleSidebar(aSidebar);
            break;
      }

   }
}

// The sort and count function, adapted from the original sort function by Torisugari.
// aType = "extension" or "theme", sortPref = true or false

function embSortAndCount(aType,aSortPref)
{
   var whichRoot = "urn:mozilla:" + aType + ":root";
   var sortTarget = function(aResource,aSortProperty)
   {
      this.resource = aResource;
      this.prop = aSortProperty.toLowerCase();
      this.toString = function(){return this.prop;};
   };

   var RDFService = Components.classes["@mozilla.org/rdf/rdf-service;1"]
               .getService(Components.interfaces.nsIRDFService);
   var Container = Components.classes["@mozilla.org/rdf/container;1"]
               .getService(Components.interfaces.nsIRDFContainer);

   var extensionDS = Components.classes["@mozilla.org/extensions/manager;1"]
               .getService(Components.interfaces.nsIExtensionManager).datasource;
   var root = RDFService.GetResource(whichRoot);
   Container.Init(extensionDS,root);

   var arc = RDFService.GetResource("http://www.mozilla.org/2004/em-rdf#name");
   var elements = Container.GetElements();
   var sortArray = [];
   try {
      while (elements.hasMoreElements())
      {
         var element=elements.getNext();
         var prop="";
         element.QueryInterface(Components.interfaces.nsIRDFResource);
         var target=extensionDS.GetTarget(element, arc ,true);
         if (target) prop = target.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
         sortArray.push(new sortTarget(element,prop));
         if (aSortPref) Container.RemoveElement(element,false);
      }
   } catch(e){}

   if (aSortPref) {
      sortArray.sort();
      for(var i in sortArray)
         Container.AppendElement(sortArray[i].resource);

// This method of updating the list is painfully slow, but is the only one I've found
// which doesn't cause problems with the count or hidden themes.
//
//      var lastSortElement = sortArray.length - 1;
//      for (var i in sortArray) {
//         var item = sortArray[lastSortElement-i].resource;
//         if (Container.IndexOf(item) > 1) {
//            Container.RemoveElement(item, true);
//            Container.InsertElementAt(item, 1, true);
//         }
//      }

   }

//   var extCount = Container.GetCount();
   var extCount = sortArray.length;
   try {
      gPref.setIntPref("extensions.embuttons.count." + aType, extCount);
   } catch(e){}

   return extCount;
}

function embBugEvade()
{
// An attempt to beat the Firefox Event handler bug - does nothing worthwhile
   try {
      var conciseMode = (gPref.getBoolPref("extensions.embuttons.conciseModeEM"));
   } catch(e){}
}



